%-- set vehicle parameters
fprintf('setting bridge parameters...\n')
EI  = 156e9; %-- flexural rigidity (N m2)
rA  = 3000;  %-- mass per unit length (kg/m)
L   = 100;   %-- bridge span length (m)
xB0 = 60;    %-- bridge entrance position (m)
enum= 20;    %-- element number

fprintf('setting vehicle parameters...\n')
ms  = 9000;  %-- sprung-mass (kg)
mu1 = 500;   %-- unsprung-mass (kg)
mu2 = 500;   %-- unsprung-mass (kg)
cs1 = 2000;  %-- sprung-damping (kg/s)
cs2 = 2000;  %-- sprung-damping (kg/s)
ks1 = 4500;  %-- sprung-stifness (N/m)=(kg/s/s)
ks2 = 4500;  %-- sprung-stifness (N/m)=(kg/s/s)
ku1 = 60000; %-- unsprung-stifness (N/m)=(kg/s/s)
ku2 = 60000; %-- unsprung-stifness (N/m)=(kg/s/s)
d1  = 1.75;  %-- distance from G to s1 (m)
d2  = 1.75;  %-- distance from G to s2 (m)
v   = 10.0;  %-- speed (m/s)
dt  = 0.001; %-- time increment
T   = 60;    %-- end time
x0  = 10;    %-- start position of G(m)
g   = -9.81; %-- gravity acceleration (m/s/s)

%-- calculate input
fprintf('calculating vehicle input...\n')
R = load('profile-1.txt'); %-- load: R(x)
x = (0:0.20:(length(R)-1)*0.20)-xB0; %-- dx=0.20(m) for R(x)
t = 0:dt:T; %-- time, dt=0.001(s)
xx= [x0+v*t+d1; x0+v*t-d2]-xB0; %-- vehicle position: x(t)
r = spline(x,R,xx); %-- spline interpolation: R(x) to r(t)=R(x(t))

%-- simulate output
fprintf('vehicle matrices...\n')
G  = 1/2;  %-- gammma coefficient for newmark-beta method
B  = 1/4;  %-- beta coefficient for newmark-beta method
TRH= 1e-6; %-- threshold
I = d1*d2*ms/(d1+d2);
Mv= [d2*ms/(d1+d2) d1*ms/(d1+d2) 0       0       ;
     I/(d1+d2)     -I/(d1+d2)    0       0       ;
     0             0             mu1     0       ;
     0             0             0       mu2    ];
Cv= [cs1           cs2           -cs1    -cs2    ;
     d1*cs1        -d2*cs2       -d1*cs1 d2*cs2  ;
     -cs1          0             cs1     0       ;
     0             -cs2          0       cs2    ];
Kv= [ks1           ks2           -ks1    -ks2    ;
     d1*ks1        -d2*ks2       -d1*ks1 d2*ks2  ;
     -ks1          0             ks1+ku1 0       ;
     0             -ks2          0       ks2+ku2];
Mp= [d2*ms/(d1+d2) 0             mu1     0       ;
     0             d1*ms/(d1+d2) 0       mu2    ];
fprintf('bridge matrices...\n')
h  = L/enum;
dof= 2*(enum+1);
Mb = zeros(2*(enum+1),2*(enum+1));
Kb = zeros(2*(enum+1),2*(enum+1));
Cb = zeros(2*(enum+1),2*(enum+1));
for ee=1:enum
    idx = 2*(ee-1)+(1:4);
    Kb(idx,idx) = Kb(idx,idx) + EI/h^3 * [12    6*h   -12   6*h   ;
                                          6*h   4*h^2 -6*h  2*h^2 ;
                                          -12   -6*h  12    -6*h  ;
                                          6*h   2*h^2 -6*h  4*h^2];
    Mb(idx,idx) = Mb(idx,idx) + rA/2 *   [1     0     0     0     ;
                                          0     0     0     0     ;
                                          0     0     1     0     ;
                                          0     0     0     0    ];
end
Cb = 0.07*Mb+0.0001*Kb;
y  = zeros(dof,length(t));
dy = zeros(dof,length(t));
ddy= zeros(dof,length(t));
mem= zeros(dof,length(t)); %-- memory

fprintf('VBI-system simulation: newmark-beta method...\n')
for count=1:100
    %-- vehicle system
    yt = zeros(2,length(t));
    for tt=1:length(t)
        LL = zeros( dof, 2 ); %-- distribtuion matrix
        for ee=1:enum
            c = xx(:,tt) > h*(ee-1) & xx(:,tt) <= h*ee;
            s = (xx(:,tt)-h*(ee-1))/h; %-- local position
            LL(ee*2-1,:) = LL(ee*2-1,:) + ( c.*(1-3*s.^2+2*s.^3) )';
            LL(ee*2  ,:) = LL(ee*2  ,:) + ( c.*(s-2*s.^2+s.^3)*h )';
            LL(ee*2+1,:) = LL(ee*2+1,:) + ( c.*(3*s.^2-2*s.^3) )';
            LL(ee*2+2,:) = LL(ee*2+2,:) + ( c.*(-s.^2+s.^3)*h )';
        end
        yt(:,tt)  = LL'*y(:,tt);
    end
    u = r+yt;
    Fv= [zeros(2,length(t)); [ku1 0; 0 ku2]*u];
    [ t, z, dz, ddz ] = newmark_beta( Mv,Cv,Kv,Fv,1/1000,zeros(4,3), G, B );

    %-- bridge system
    P = Mp*(repmat(g,4,length(t))-ddz);
    Fb= zeros(dof,length(t));
    for tt=1:length(t)
        LL = zeros( dof, 2 ); %-- distribtuion matrix
        for ee=1:enum
            c = xx(:,tt) > h*(ee-1) & xx(:,tt) <= h*ee;
            s = (xx(:,tt)-h*(ee-1))/h; %-- local position
            LL(ee*2-1,:) = LL(ee*2-1,:) + ( c.*(1-3*s.^2+2*s.^3) )';
            LL(ee*2  ,:) = LL(ee*2  ,:) + ( c.*(s-2*s.^2+s.^3)*h )';
            LL(ee*2+1,:) = LL(ee*2+1,:) + ( c.*(3*s.^2-2*s.^3) )';
            LL(ee*2+2,:) = LL(ee*2+2,:) + ( c.*(-s.^2+s.^3)*h )';
        end
        Fb(:,tt)  = LL*P(:,tt);
    end
    %-- boundary condition
    bcd = [1;dof-1];
    bca = repmat([0;0],1,length(t));
    bcv = repmat([0;0],1,length(t));
    bcy = repmat([0;0],1,length(t));
    idx = 1:dof;
    idx(bcd)=[];
    Mb0 = Mb(idx,idx);
    Cb0 = Cb(idx,idx);
    Kb0 = Kb(idx,idx);
    Fb0 = Fb(idx,:) - Mb(idx,bcd)*bca - Cb(idx,bcd)*bcv - Kb(idx,bcd)*bcy;
    [ t, y0, dy0, ddy0 ] = newmark_beta( Mb0,Cb0,Kb0,Fb0,1/1000,zeros(dof-2,3), G, B );
    y(idx,:)   = y0;
    dy(idx,:)  = dy0;
    ddy(idx,:) = ddy0;
    
    ERR=norm(ddy-mem)/norm(mem);
    if ERR<TRH
        break;
    end
    fprintf([num2str(count,'%0.3d') ': ERR=' num2str(ERR) '\n'])
    mem = ddy;
end

%-- visualization
fprintf('outputting the simulation results...\n')
figure(1)
subplot(3,1,1); plot(t,u(1:2,:),'-r');hold on
subplot(3,1,1); plot(t,r(1:2,:),'-b');hold off
subplot(3,1,2); plot(t,z(1:2,:),'-r')
subplot(3,1,3); plot(t,z(3:4,:),'-r')

%-- animation
figure(2);set(gcf, 'Unit', 'pixel', 'Position',[120, 280, 800, 300]);
writerObj = VideoWriter('ex_04.avi','Motion JPEG AVI'); %-- setting movie class
writerObj.FrameRate = 20; %-- flame rate (slide/sec)
open(writerObj); %-- initialization of movie object
for tt=1:100:length(t)
    plot(xx(:,1:tt)',u(:,1:tt)','b-');hold on
    plot(xx(:,tt)',u(:,tt)','bo');
    plot(xx(:,1:tt)',z(1:2,1:tt)'+0.02,'r-');
    plot(xx(:,tt)',z(1:2,tt)'+0.02,'ro');
    plot(xx(:,1:tt)',z(3:4,1:tt)'+0.01,'r-');
    plot(xx(:,tt)',z(3:4,tt)'+0.01,'ro');
    plot([0 0],[-10 10],'k:');
    plot([L L],[-10 10],'k:');
    plot([0 L],[0 0],'k-');
    plot([xx(1,tt)-40 0],[0 0],'k:');
    plot([0 xx(1,tt)+10],[0 0],'k:');
    hold off
    xlim([xx(1,tt)-60 xx(1,tt)+10]);ylim([-0.020 0.025]);
    frame = getframe(figure(2)); %-- get the figure as a frame of the animation
    writeVideo(writerObj,frame); %-- add the gotten frame to the movie object
end
close(writerObj); %-- end of animation


